/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_register.c,v $
 *
 * $Author: mr $
 * $Date: 2004/01/26 23:54:07 $
 * 
 * $Revision: 1.2 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include "gj_register.h"
#include "gj_parser.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"

static void gj_register_init();

/* registration list */
static gboolean gj_register_list_add(GjRegistration reg);
static gboolean gj_register_list_del(GjRegistration reg);

/* registration item */
static void gj_register_destroy(GjRegistration reg);

/* parsing functions */
static void gj_register_cb(GjInfoquery iq, GjInfoquery iq_related);
static GjRegistration gj_register_parse(xmlNodePtr parent);


struct t_registration
{
  gchar *id;
  gchar *to;
  gchar *from;

  gchar *instructions;

  gchar *username;
  gchar *password;
  gchar *email;

  gboolean require_username;
  gboolean require_password;
  gboolean require_email;
  gboolean require_nickname;

  gchar *key;

  gpointer user_data;

  GjRegistrationCallback cb; 
};


static GList *registration_list = NULL;
static GjRegistrationRequestCallback registration_cb = NULL;
static gpointer user_data = NULL;


static void gj_register_init()
{
  static gboolean initiated = FALSE;
  
  if(initiated == TRUE)
    return;

  initiated = TRUE;
}

static gboolean gj_register_list_add(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  registration_list = g_list_append(registration_list, reg);
  return TRUE;
}

static gboolean gj_register_list_del(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  registration_list = g_list_remove(registration_list, reg);
  return TRUE;
}

GList *gj_register_list_get()
{
  return registration_list;
}

/* 
 * item functions
 */
GjRegistration gj_register_new()
{
  GjRegistration reg = NULL;

  gj_register_init();

  if((reg = g_new(struct t_registration,1)))
    {
      reg->id = NULL;
      reg->to = NULL;
      reg->from = NULL;
      
      reg->instructions = NULL;

      reg->username = NULL;
      reg->password = NULL;
      reg->email = NULL;

      reg->require_username = FALSE;
      reg->require_password = FALSE;
      reg->require_email = FALSE;
      reg->require_nickname = FALSE;

      reg->key = NULL;
      
      reg->cb = NULL;

      gj_register_list_add(reg);
    }
  
  return reg;  
}

void gj_register_free(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return;
    }

  /* free mem */
  gj_register_list_del(reg);  

  /* destroy members */
  gj_register_destroy(reg);

  /* free structure */
  g_free(reg);
}

static void gj_register_destroy(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return;
    }
  
  if(reg->id != NULL)
    {
      g_free(reg->id);
      reg->id = NULL;
    }

  if(reg->to != NULL)
    {
      g_free(reg->to);
      reg->to = NULL;
    }

  if(reg->from != NULL)
    {
      g_free(reg->from);
      reg->from = NULL;
    }

  if(reg->instructions != NULL)
    {
      g_free(reg->instructions);
      reg->instructions = NULL;
    }

  if(reg->username != NULL)
    {
      g_free(reg->username);
      reg->username = NULL;
    }

  if(reg->password != NULL)
    {
      g_free(reg->password);
      reg->password = NULL;
    }

  if(reg->email != NULL)
    {
      g_free(reg->email);
      reg->email = NULL;
    }

  reg->require_username = FALSE;
  reg->require_password = FALSE;
  reg->require_email = FALSE;
  reg->require_nickname = FALSE;
  
  if(reg->key != NULL)
    {
      g_free(reg->key);
      reg->key = NULL;
    }

  reg->cb = NULL;
}

/*
 * find 
 */
GjRegistration gj_register_find_by_id(const gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("%s: id was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(registration_list);index++)
    {
      GjRegistration reg  = g_list_nth_data(registration_list, index);

      if(reg == NULL) continue;
      if(g_strcasecmp(reg->id, id) == 0) return reg;
    }

  return NULL;
}

GjRegistration gj_register_find_by_to(const gchar *to)
{
  gint index = 0;

  if(to == NULL)
    {
      g_warning("%s: to was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(registration_list);index++)
    {
      GjRegistration reg  = g_list_nth_data(registration_list, index);

      if(reg == NULL) continue;
      if(g_strcasecmp(reg->to, to) == 0) return reg;
    }

  return NULL;
}

GjRegistration gj_register_find_by_from(const gchar *from)
{
  gint index = 0;

  if(from == NULL)
    {
      g_warning("%s: from was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(registration_list);index++)
    {
      GjRegistration reg  = g_list_nth_data(registration_list, index);

      if(reg == NULL) continue;
      if(g_strcasecmp(reg->from, from) == 0) return reg;
    }

  return NULL;
}

/*
 * member functions  
 */

/* sets */
gboolean gj_register_set_id(GjRegistration reg, const gchar *id)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("%s: id was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->id != NULL)
    {
      g_free(reg->id);
      reg->id = NULL;
    }
  
  reg->id = g_strdup(id);
  return TRUE;
}

gboolean gj_register_set_to(GjRegistration reg, const gchar *to)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(to == NULL)
    {
      g_warning("%s: to was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->to != NULL)
    {
      g_free(reg->to);
      reg->to = NULL;
    }
  
  reg->to = g_strdup(to);
  return TRUE;
}

gboolean gj_register_set_from(GjRegistration reg, const gchar *from)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(from == NULL)
    {
      g_warning("%s: from was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->from != NULL)
    {
      g_free(reg->from);
      reg->from = NULL;
    }
  
  reg->from = g_strdup(from);
  return TRUE;
}

gboolean gj_register_set_instructions(GjRegistration reg, const gchar *instructions)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(instructions == NULL)
    {
      g_warning("%s: instructions was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->instructions != NULL)
    {
      g_free(reg->instructions);
      reg->instructions = NULL;
    }
  
  reg->instructions = g_strdup(instructions);
  return TRUE;
}

gboolean gj_register_set_username(GjRegistration reg, const gchar *username)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(username == NULL)
    {
      g_warning("%s: username was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->username != NULL)
    {
      g_free(reg->username);
      reg->username = NULL;
    }
  
  reg->username = g_strdup(username);
  return TRUE;
}

gboolean gj_register_set_password(GjRegistration reg, const gchar *password)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(password == NULL)
    {
      g_warning("%s: password was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->password != NULL)
    {
      g_free(reg->password);
      reg->password = NULL;
    }
  
  reg->password = g_strdup(password);
  return TRUE;
}

gboolean gj_register_set_email(GjRegistration reg, const gchar *email)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(email == NULL)
    {
      g_warning("%s: email was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->email != NULL)
    {
      g_free(reg->email);
      reg->email = NULL;
    }
  
  reg->email = g_strdup(email);
  return TRUE;
}

gboolean gj_register_set_require_username(GjRegistration reg, gboolean required)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }
  
  reg->require_username = required;
  return TRUE;
}

gboolean gj_register_set_require_password(GjRegistration reg, gboolean required)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }
  
  reg->require_password = required;
  return TRUE;
}

gboolean gj_register_set_require_email(GjRegistration reg, gboolean required)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }
  
  reg->require_email = required;
  return TRUE;
}

gboolean gj_register_set_require_nickname(GjRegistration reg, gboolean required)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }
  
  reg->require_nickname = required;
  return TRUE;
}

gboolean gj_register_set_key(GjRegistration reg, const gchar *key)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  if(key == NULL)
    {
      g_warning("%s: key was NULL", __FUNCTION__);
      return FALSE;
    }

  if(reg->key != NULL)
    {
      g_free(reg->key);
      reg->key = NULL;
    }
  
  reg->key = g_strdup(key);
  return TRUE;
}

gboolean gj_register_set_callback(GjRegistration reg, GjRegistrationCallback cb)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  reg->cb = cb;
  return TRUE;
}

/* gets */
const gchar *gj_register_get_id(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->id;
}

const gchar *gj_register_get_to(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->to;
}

const gchar *gj_register_get_from(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->from;
}

const gchar *gj_register_get_instructions(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->instructions;
}

const gchar *gj_register_get_password(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->password;
}

const gchar *gj_register_get_email(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->email;
}

gboolean gj_register_get_require_username(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  return reg->require_username;
}

gboolean gj_register_get_require_password(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  return reg->require_password;
}

gboolean gj_register_get_require_email(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  return reg->require_email;
}

gboolean gj_register_get_require_nickname(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return FALSE;
    }

  return reg->require_nickname;
}

const gchar *gj_register_get_key(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->key;
}

GjRegistrationCallback gj_register_get_callback(GjRegistration reg)
{
  if(reg == NULL)
    {
      g_warning("%s: registration was NULL", __FUNCTION__);
      return NULL;
    }

  return reg->cb;
}

/*
 *
 * SERVICES
 *
 */
gboolean gj_register_request_requirements(const gchar *server, 
					  GjRegistrationRequestCallback cb)
{
  if(server == NULL)
    {
      g_warning("%s: server was NULL", __FUNCTION__);
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  registration_cb = cb;

  /* do request */
  gj_iq_request_registration_requirements(server, gj_register_cb, NULL);

  return TRUE;
}

gboolean gj_register_request(const gchar *server, 
			     GjRegistrationRequestCallback cb, 
			     const gchar *key, 
			     const gchar *username, 
			     const gchar *password, 
			     const gchar *email, 
			     const gchar *nickname)
{
  if(server == NULL)
    {
      g_warning("%s: server was NULL", __FUNCTION__);
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  registration_cb = cb;

  /* do request */
  gj_iq_request_registration(server, gj_register_cb, 
			     key, username, password, email, nickname, NULL);

  return TRUE;
}

static void gj_register_cb(GjInfoquery iq, GjInfoquery iq_related)
{
  GjRegistration reg = NULL;

  gint error_code = 0;
  gchar *error_reason = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  if(!iq)
    return;

  if(registration_cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return;
    }

  node = gj_iq_get_xml(iq);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);
	  return;
	}

      errorCode = gj_parser_find_by_attr_and_ret(node, (guchar*)"code");
      errorMessage = gj_parser_find_by_node_and_ret(node, (guchar*)"error");

      error_code = atoi((gchar*)errorCode);
      error_reason = g_strdup((gchar*)errorMessage);
    }

  /* parse list */
  if((reg = gj_register_parse(node)) == NULL)
    {
      g_warning("%s: could not parse xml node", __FUNCTION__);
      return;
    }

  /* return list */
  (registration_cb)(reg, error_code, error_reason);

  /* clean up */
  if(error_reason != NULL) g_free(error_reason);
}

static GjRegistration gj_register_parse(xmlNodePtr parent)
{
  xmlChar *tmp = NULL;
  GjRegistration reg = NULL;

  if(parent == NULL)
    {
      g_warning("%s: xml node was NULL", __FUNCTION__);
      return NULL;
    }

  /* create new roster item */
  if((reg = gj_register_new()) == NULL)
    {
      g_warning("%s: could not create registration", __FUNCTION__);
      return NULL;
    }

  /* get important information */
  if((tmp = gj_parser_find_by_attr_and_ret(parent, (guchar*)"id")) != NULL)
    gj_register_set_id(reg, (gchar*)tmp);
  if((tmp = gj_parser_find_by_attr_and_ret(parent, (guchar*)"to")) != NULL)
    gj_register_set_to(reg, (gchar*)tmp);
  if((tmp = gj_parser_find_by_attr_and_ret(parent, (guchar*)"from")) != NULL)
    gj_register_set_from(reg, (gchar*)tmp);
  
  /* get instructions */
  if((tmp = gj_parser_find_by_node_and_ret(parent, (guchar*)"instructions")) != NULL)
    gj_register_set_instructions(reg, (gchar*)tmp);

  /* get user/pass... */
  if(gj_parser_find_by_node(parent, (guchar*)"username") != NULL)
    gj_register_set_require_username(reg, TRUE);
  if(gj_parser_find_by_node(parent, (guchar*)"password") != NULL)
    gj_register_set_require_password(reg, TRUE);
  if(gj_parser_find_by_node(parent, (guchar*)"email") != NULL)
    gj_register_set_require_email(reg, TRUE);
  if(gj_parser_find_by_node(parent, (guchar*)"nickname") != NULL)
    gj_register_set_require_nickname(reg, TRUE);

  if((tmp = gj_parser_find_by_node_and_ret(parent, (guchar*)"key")) != NULL)
    gj_register_set_key(reg, (gchar*)tmp);

  return reg;
}

#ifdef __cplusplus
}
#endif
